;  SER_IQFreqSource.ASM      G4JNT  Nov 2011

;Sampling rate Fosc / (4 * IntDivider)


    LIST P = 16F870
    INCLUDE "P16F870.INC"

    __config    0x3D72      ;LV Programming off , OSC HS Mode

    __idlocs    0x813E

	errorlevel 2

	#define		SClk		PORTA, 0
	#define		SDat		PORTA, 1
	#define		SStr		PORTA, 2
	#define		TESTSTRB	PORTA, 3

	DataFlag	=	0xF8
	IntDivider 	=	d'60'	;Every int, prog TMR0 prog with     ;NNN = 256 - (IntDivider - Overhead) / Prescale
									    ; (calculated in the movlw instruction below)
    org     0

	clrf	INTCON          ;
	goto    startup         ;jump to main code
;------------------ Interrupt routine, get here 4 cycles after TMR overflow --
    org 4
ints      	  ;Interupt service at Fosc / 216.  NO REGISTERS are saved,
     	bcf     INTCON , T0IF   	;  every IntDivider * prescale clocks = sampling rate. 
        movwf   Wreg        ;Save W
	swapf   STATUS , W      	; nb.  movwf does not affect any STATUS bits
	movwf   SaveSts         	;Stave STATUS, without affecting it

	nop
        bsf	TESTSTRB        	;Spare instruction needed to make up 12 clocks total up to point of reloading TMR0
        movlw   d'256'-IntDivider/2+6          ; = NNN:  
        movwf   TMR0            	;Timer restarts counting 2 cycles after here.
                                	;To here takes 8 clocks (with the extra instruction)
                            
 ;Below here is not time-critical, but must not overrun the whole interrupt service.

	movf	D2, W			;Do this first so there is no jittering due to variable add routine times.
	call	SineTable		;Takes 5 clocks incl call and return
	movwf	PortCache		;save for PORTB

	movf	D2, W
	addlw	0x40			;90 degree phase shift
	call	SineTable

	movwf	PORTC
	movf	PortCache, W
	movwf	PORTB			;Two clock cycles phase offset
					;~~~~~~~~  29 clock cycles used up to here
	movf	F0, W
	addwf	D0
	btfss	STATUS, C
	goto	Add1Done
	incf	D1
	btfsc	STATUS, Z
	incf	D2

Add1Done
	movf	F1, W
	addwf	D1
	btfss	STATUS, C
	goto	Add2Done	
	incf	D2

Add2Done
	movf	F2, W
	addwf	D2		;D0-2 updated ready for next pass through

OutInt			
	bcf	TESTSTRB	
	swapf   SaveSts , W     ;Restore STATUS
	movwf   STATUS
	swapf   Wreg            ;Restore W
	swapf   Wreg , W        ;Can't use movf as it  affects Z bit
	retfie			;Looks to be about 50 clocks; maximum 54

;=====================================
startup 
	clrf	PORTA

        bcf     STATUS, RP1
        bsf     STATUS, RP0      ;ram page 1

	movlw	0x06		 ;Set all PORTA as digital - coz it defaults to analogue. WHY ?????????!!!!!!!!!!!!!!!!
	movwf	ADCON1

        movlw   b'00000111'      ;A0-2 Serial inputs, A3 test strobe out
        movwf   PORTA           
        movlw   b'00000000'     
        movwf   PORTB           
        movlw   b'00000000'     
        movwf   PORTC           
	
        movlw   b'11000000'     ;Internal input, Prescalar /2 no pull-ups
        movwf   OPTION_REG

	bcf	STATUS, RP0

	movlw	HIGH(TabContents)
	movwf	PCLATH

	movlw	0x40
	movwf	F2
	clrf	F1
	clrf	F0

	bsf     INTCON, T0IE   
	bsf	INTCON, GIE

;------------------
MainLoop			
	call	GetSerWord
	goto	MainLoop
;------------------
GetSerWord		;Must be able to work with interrupt firing continuously

SerLoop1			;Check for initial low on SClk
	btfsc	SClk
	goto	SerLoop1
	call	ShortDel

SerLoop2			;Wait for clock to go high which defines the centre of data bit
	btfsc	SStr		; or look for Strobe signal
	goto	TransferData

	btfss	SClk
	goto	SerLoop2

	btfsc	SDat		;Should be sitting right in middle of the data bit
	bsf	STATUS, C
	btfss	SDat	
	bcf	STATUS, C
	rlf	K0
	rlf	K1
	rlf	K2
	rlf	K3

SerLoop3			;Now wait for the clock to go low again
	btfsc	SClk
	goto	SerLoop1

;.........
TransferData				;K3 - K0 should now have correct data

	movlw	DataFlag
	subwf	K3
	btfss	STATUS, Z
	goto	WaitStrLow		;If header flag doesn't match, ignore and get out

	bcf	INTCON, GIE	;Stop interrupt briefly while transferring frequency data
	movf	K0, W
	movwf	F0
	movf	K1, W
	movwf	F1
	movf	K2, W
	movwf	F2
	bsf	INTCON, GIE	

WaitStrLow
	btfsc	SStr
	goto	WaitStrLow

	return
;---------------------
ShortDel			;
	movlw	d'50'
	movwf	DelCount
DelLoop
	nop
	nop
	decfsz	DelCount
	goto	DelLoop
	return
;---------------------

;---------------------

  org 0x6FF
SineTable
		;Adjust so TabContents falls EXACTLY on the next page boundary
	addwf	PCL
TabContents
    dt    0x80,0x83,0x86,0x89,0x8C,0x8F,0x92,0x95
    dt    0x98,0x9C,0x9F,0xA2,0xA5,0xA8,0xAB,0xAE
    dt    0xB0,0xB3,0xB6,0xB9,0xBC,0xBF,0xC1,0xC4
    dt    0xC7,0xC9,0xCC,0xCE,0xD1,0xD3,0xD5,0xD8
    dt    0xDA,0xDC,0xDE,0xE0,0xE2,0xE4,0xE6,0xE8
    dt    0xEA,0xEC,0xED,0xEF,0xF0,0xF2,0xF3,0xF5
    dt    0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFC
    dt    0xFD,0xFE,0xFE,0xFF,0xFF,0xFF,0xFF,0xFF
    dt    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFE,0xFE
    dt    0xFD,0xFC,0xFC,0xFB,0xFA,0xF9,0xF8,0xF7
    dt    0xF6,0xF5,0xF3,0xF2,0xF0,0xEF,0xED,0xEC
    dt    0xEA,0xE8,0xE6,0xE4,0xE2,0xE0,0xDE,0xDC
    dt    0xDA,0xD8,0xD5,0xD3,0xD1,0xCE,0xCC,0xC9
    dt    0xC7,0xC4,0xC1,0xBF,0xBC,0xB9,0xB6,0xB3
    dt    0xB0,0xAE,0xAB,0xA8,0xA5,0xA2,0x9F,0x9C
    dt    0x98,0x95,0x92,0x8F,0x8C,0x89,0x86,0x83
    dt    0x7F,0x7C,0x79,0x76,0x73,0x70,0x6D,0x6A
    dt    0x67,0x63,0x60,0x5D,0x5A,0x57,0x54,0x51
    dt    0x4F,0x4C,0x49,0x46,0x43,0x40,0x3E,0x3B
    dt    0x38,0x36,0x33,0x31,0x2E,0x2C,0x2A,0x27
    dt    0x25,0x23,0x21,0x1F,0x1D,0x1B,0x19,0x17
    dt    0x15,0x13,0x12,0x10,0x0F,0x0D,0x0C,0x0A
    dt    0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x03
    dt    0x02,0x01,0x01,0x00,0x00,0x00,0x00,0x00
    dt    0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01
    dt    0x02,0x03,0x03,0x04,0x05,0x06,0x07,0x08
    dt    0x09,0x0A,0x0C,0x0D,0x0F,0x10,0x12,0x13
    dt    0x15,0x17,0x19,0x1B,0x1D,0x1F,0x21,0x23
    dt    0x25,0x27,0x2A,0x2C,0x2E,0x31,0x33,0x36
    dt    0x38,0x3B,0x3E,0x40,0x43,0x46,0x49,0x4C
    dt    0x4F,0x51,0x54,0x57,0x5A,0x5D,0x60,0x63
    dt    0x67,0x6A,0x6D,0x70,0x73,0x76,0x79,0x7C

;--------------------------------
    Cblock 0x20
	Wreg, SaveSts
	Temp
	PortCache
	Flags
	DelCount
	Counter
	D0, D1, D2
	F0, F1, F2		;Working frequency
	K0, K1, K2, K3		;Received data, cache for new frequency

    endc

	end















